package com.usky.system.controller.web;

import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.JwtUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.redis.core.RedisHelper;
import com.usky.common.security.auth.AuthUtil;
import com.usky.common.security.service.TokenService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.domain.SysPerson;
import com.usky.system.domain.SysUser;
import com.usky.system.domain.SysUserVO;
import com.usky.system.model.LoginUser;
import com.usky.system.service.ISysPermissionService;
import com.usky.system.service.ISysUserService;
import com.usky.system.service.SysLoginService;
import com.usky.system.service.SysPersonService;
import com.usky.system.service.vo.LoginBody;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * token 控制
 *
 * @author ruoyi
 */
@RestController
public class TokenController {
    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysLoginService sysLoginService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysPermissionService permissionService;

    @Autowired
    private SysPersonService sysPersonService;

    @PostMapping("login")
    public ApiResult<?> login(@RequestBody LoginBody form) {
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword(), form.getTenantId());
        // 获取登录token
        return ApiResult.success(tokenService.createToken(userInfo));
    }


    @PostMapping("appLogin")
    public ApiResult<?> appLogin(@RequestBody LoginBody form) {
//        String phone = null;
        // 用户登录
        SysUser userInfo = sysLoginService.appLogin(form.getUsername(), form.getPassword(), form.getTenantId(), form.getPhone(), form.getVerify());
        LoginUser SysUser = new LoginUser();
        SysUserVO sysUserVO = BeanMapperUtils.map(userInfo, SysUserVO.class);
        SysUser.setSysUser(sysUserVO);
        SysPerson sysPerson = sysPersonService.getsysPerson(userInfo.getUserId());
        SysUser.setSysPerson(sysPerson);
        // 获取登录token
        return ApiResult.success(tokenService.createToken(SysUser));
    }


    //手机端发送扫码信息，二维码编号，用户名信息
    @GetMapping("getQrCodeResult")
    public ApiResult<String> getQrCodeResult(@RequestParam String qrCode){
        String qrCodeResult = sysLoginService.getQrCodeResult(qrCode);
        return ApiResult.success(qrCodeResult);
    }

    //PC通过二维码编号获取用户登录授权

    //



//    @PostMapping("login1")
//    public ApiResult<?> login1(@RequestBody LoginBody form) {
//        // 用户登录
//        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword(), form.getTenantId());
//        // 获取登录token
//        return ApiResult.success(tokenService.createToken(userInfo));
//    }

    @DeleteMapping("logout")
    public ApiResult<?> logout(HttpServletRequest request) {
        String token = SecurityUtils.getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            String username = JwtUtils.getUserName(token);
            // 删除用户缓存记录
            AuthUtil.logoutByToken(token);
            // 记录用户退出日志
            sysLoginService.logout(username);
        }
        return ApiResult.success();
    }

    @PostMapping("refresh")
    public ApiResult<?> refresh(HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (Objects.nonNull(loginUser)) {
            // 刷新令牌有效期
            tokenService.refreshToken(loginUser);
            return ApiResult.success();
        }
        return ApiResult.success();
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public ApiResult getInfo() {
        Long userId = SecurityUtils.getUserId();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(userId);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(userId);
        Map<String, Object> ajax = new HashMap<>();
        ajax.put("user", sysUserService.selectUserById(userId));
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ApiResult.success(ajax);
    }


//    @PostMapping("register")
//    public ApiResult<?> register(@RequestBody RegisterBody registerBody)
//    {
//        // 用户注册
//        sysLoginService.register(registerBody.getUsername(), registerBody.getPassword());
//        return ApiResult.success();
//    }

    @GetMapping("/qrCodeSend")
    public ApiResult<Void> qrCodeSend(@RequestParam String userName, @RequestParam String qrCode,@RequestParam Integer tenantId) {
        sysLoginService.qrCodeSend(userName,qrCode,tenantId);
        return ApiResult.success();
    }
}
